@@ -1,13 +1,13 @@
 from __future__ import annotations
 
-from abc import ABC, abstractmethod
+import itertools
+from abc import ABC
 from collections import OrderedDict, deque
 from collections.abc import (
     AsyncGenerator,
     AsyncIterator,
     Awaitable,
     Callable,
-    Collection,
     Generator,
     Iterable,
     Iterator,
@@ -29,39 +29,39 @@ from inspect import signature as inspect_signature
 from logging import Logger, getLogger
 from types import MethodType
 from typing import (
+    TYPE_CHECKING,
     Any,
     AsyncContextManager,
     ClassVar,
     ContextManager,
     Literal,
     NamedTuple,
-    Protocol,
     Self,
     overload,
-    runtime_checkable,
 )
 
+from type_analyzer import MatchingTypesConfig, iter_matching_types, matching_types
+
 from injection._core.common.asynchronous import (
-    AsyncCaller,
     Caller,
+    HiddenCaller,
     SimpleAwaitable,
-    SyncCaller,
 )
 from injection._core.common.event import Event, EventChannel, EventListener
 from injection._core.common.invertible import Invertible, SimpleInvertible
 from injection._core.common.key import new_short_key
-from injection._core.common.lazy import Lazy, alazy, lazy
+from injection._core.common.lazy import Lazy
 from injection._core.common.threading import get_lock
 from injection._core.common.type import (
     InputType,
     TypeInfo,
     get_return_types,
     get_yield_hint,
-    standardize_types,
 )
 from injection._core.injectables import (
     AsyncCMScopedInjectable,
     CMScopedInjectable,
+    ConstantInjectable,
     Injectable,
     ScopedInjectable,
     ScopedSlotInjectable,
@@ -70,6 +70,18 @@ from injection._core.injectables import (
     SingletonInjectable,
     TransientInjectable,
 )
+from injection._core.locator import (
+    DynamicInjectableBroker,
+    InjectableBroker,
+    InjectableFactory,
+    InjectionProvider,
+    Locator,
+    Mode,
+    ModeStr,
+    Recipe,
+    StaticInjectableBroker,
+    Updater,
+)
 from injection._core.slots import SlotKey
 from injection.exceptions import (
     ModuleError,
@@ -84,25 +96,6 @@ Events
 """
 
 
-@dataclass(frozen=True, slots=True)
-class LocatorEvent(Event, ABC):
-    locator: Locator
-
-
-@dataclass(frozen=True, slots=True)
-class LocatorDependenciesUpdated[T](LocatorEvent):
-    classes: Collection[InputType[T]]
-    mode: Mode
-
-    def __str__(self) -> str:
-        length = len(self.classes)
-        formatted_types = ", ".join(f"`{cls}`" for cls in self.classes)
-        return (
-            f"{length} dependenc{'ies' if length > 1 else 'y'} have been "
-            f"updated{f': {formatted_types}' if formatted_types else ''}."
-        )
-
-
 @dataclass(frozen=True, slots=True)
 class ModuleEvent(Event, ABC):
     module: Module
@@ -116,15 +109,17 @@ class ModuleEventProxy(ModuleEvent):
         return f"`{self.module}` has propagated an event: {self.origin}"
 
     @property
-    def history(self) -> Iterator[Event]:
-        if isinstance(self.event, ModuleEventProxy):
-            yield from self.event.history
-
-        yield self.event
+    def origin(self) -> Event:
+        reversed_proxy_history = reversed(tuple(self.proxy_history))
+        return next(reversed_proxy_history, self).event
 
     @property
-    def origin(self) -> Event:
-        return next(self.history)
+    def proxy_history(self) -> Iterator[ModuleEventProxy]:
+        event = self.event
+
+        if isinstance(event, ModuleEventProxy):
+            yield event
+            yield from event.proxy_history
 
 
 @dataclass(frozen=True, slots=True)
@@ -156,208 +151,6 @@ class ModulePriorityUpdated(ModuleEvent):
         )
 
 
-@dataclass(frozen=True, slots=True)
-class UnlockCalled(Event):
-    def __str__(self) -> str:
-        return "An `unlock` method has been called."
-
-
-"""
-Broker
-"""
-
-
-@runtime_checkable
-class Broker(Protocol):
-    __slots__ = ()
-
-    @abstractmethod
-    def __getitem__[T](self, cls: InputType[T], /) -> Injectable[T]:
-        raise NotImplementedError
-
-    @abstractmethod
-    def __contains__(self, cls: InputType[Any], /) -> bool:
-        raise NotImplementedError
-
-    @property
-    @abstractmethod
-    def is_locked(self) -> bool:
-        raise NotImplementedError
-
-    @abstractmethod
-    def unsafe_unlocking(self) -> None:
-        raise NotImplementedError
-
-    @abstractmethod
-    async def all_ready(self) -> None:
-        raise NotImplementedError
-
-
-"""
-Locator
-"""
-
-
-class Mode(StrEnum):
-    FALLBACK = "fallback"
-    NORMAL = "normal"
-    OVERRIDE = "override"
-
-    @property
-    def rank(self) -> int:
-        return tuple(type(self)).index(self)
-
-    @classmethod
-    def get_default(cls) -> Mode:
-        return cls.NORMAL
-
-
-type ModeStr = Literal["fallback", "normal", "override"]
-
-type InjectableFactory[T] = Callable[[Caller[..., T]], Injectable[T]]
-
-
-class Record[T](NamedTuple):
-    injectable: Injectable[T]
-    mode: Mode
-
-
-@dataclass(repr=False, eq=False, kw_only=True, slots=True)
-class Updater[T]:
-    classes: Iterable[InputType[T]]
-    injectable: Injectable[T]
-    mode: Mode
-
-    def make_record(self) -> Record[T]:
-        return Record(self.injectable, self.mode)
-
-    @classmethod
-    def with_basics(
-        cls,
-        on: TypeInfo[T],
-        /,
-        injectable: Injectable[T],
-        mode: Mode | ModeStr,
-    ) -> Self:
-        return cls(
-            classes=get_return_types(on),
-            injectable=injectable,
-            mode=Mode(mode),
-        )
-
-
-@dataclass(repr=False, frozen=True, slots=True)
-class Locator(Broker):
-    __records: dict[InputType[Any], Record[Any]] = field(
-        default_factory=dict,
-        init=False,
-    )
-    __channel: EventChannel = field(
-        default_factory=EventChannel,
-        init=False,
-    )
-
-    def __getitem__[T](self, cls: InputType[T], /) -> Injectable[T]:
-        for input_class in self.__standardize_inputs((cls,)):
-            try:
-                record = self.__records[input_class]
-            except KeyError:
-                continue
-
-            return record.injectable
-
-        raise NoInjectable(cls)
-
-    def __contains__(self, cls: InputType[Any], /) -> bool:
-        return any(
-            input_class in self.__records
-            for input_class in self.__standardize_inputs((cls,))
-        )
-
-    @property
-    def is_locked(self) -> bool:
-        return any(injectable.is_locked for injectable in self.__injectables)
-
-    @property
-    def __injectables(self) -> frozenset[Injectable[Any]]:
-        return frozenset(record.injectable for record in self.__records.values())
-
-    def update[T](self, updater: Updater[T]) -> Self:
-        updater = self.__update_preprocessing(updater)
-        record = updater.make_record()
-        records = dict(self.__prepare_for_updating(updater.classes, record))
-
-        if records:
-            event = LocatorDependenciesUpdated(self, records.keys(), record.mode)
-
-            with self.dispatch(event):
-                self.__records.update(records)
-
-        return self
-
-    def unsafe_unlocking(self) -> None:
-        for injectable in self.__injectables:
-            injectable.unlock()
-
-    async def all_ready(self) -> None:
-        for injectable in self.__injectables:
-            if injectable.is_locked:
-                continue
-
-            with suppress(SkipInjectable):
-                await injectable.aget_instance()
-
-    def add_listener(self, listener: EventListener) -> Self:
-        self.__channel.add_listener(listener)
-        return self
-
-    def dispatch(self, event: Event) -> ContextManager[None]:
-        return self.__channel.dispatch(event)
-
-    def __prepare_for_updating[T](
-        self,
-        classes: Iterable[InputType[T]],
-        record: Record[T],
-    ) -> Iterator[tuple[InputType[T], Record[T]]]:
-        for cls in classes:
-            try:
-                existing = self.__records[cls]
-            except KeyError:
-                ...
-            else:
-                if not self.__keep_new_record(record, existing, cls):
-                    continue
-
-            yield cls, record
-
-    @staticmethod
-    def __keep_new_record[T](
-        new: Record[T],
-        existing: Record[T],
-        cls: InputType[T],
-    ) -> bool:
-        new_mode, existing_mode = new.mode, existing.mode
-
-        if new_mode == Mode.OVERRIDE:
-            return True
-
-        elif new_mode == existing_mode:
-            raise RuntimeError(f"An injectable already exists for the class `{cls}`.")
-
-        return new_mode.rank > existing_mode.rank
-
-    @staticmethod
-    def __standardize_inputs[T](
-        classes: Iterable[InputType[T]],
-    ) -> Iterator[InputType[T]]:
-        return standardize_types(*classes, with_origin=True)
-
-    @staticmethod
-    def __update_preprocessing[T](updater: Updater[T]) -> Updater[T]:
-        updater.classes = frozenset(standardize_types(*updater.classes))
-        return updater
-
-
 """
 Module
 """
@@ -380,11 +173,10 @@ type ContextManagerLikeRecipe[**P, T] = (
 type GeneratorRecipe[**P, T] = (
     Callable[P, Generator[T, Any, Any]] | Callable[P, AsyncGenerator[T, Any]]
 )
-type Recipe[**P, T] = Callable[P, T] | Callable[P, Awaitable[T]]
 
 
 @dataclass(eq=False, frozen=True, slots=True)
-class Module(Broker, EventListener):
+class Module(EventListener, InjectionProvider):  # type: ignore[misc]
     name: str = field(default_factory=lambda: f"anonymous@{new_short_key()}")
     __channel: EventChannel = field(
         default_factory=EventChannel,
@@ -413,23 +205,26 @@ class Module(Broker, EventListener):
         self.__locator.add_listener(self)
 
     def __getitem__[T](self, cls: InputType[T], /) -> Injectable[T]:
-        for broker in self.__brokers:
-            with suppress(KeyError):
-                return broker[cls]
+        key_types = self.__matching_key_types(cls)
+
+        for locator in self._iter_locators():
+            for key_type in key_types:
+                with suppress(KeyError):
+                    return locator.request(key_type, self)
 
         raise NoInjectable(cls)
 
     def __contains__(self, cls: InputType[Any], /) -> bool:
-        return any(cls in broker for broker in self.__brokers)
+        key_types = self.__matching_key_types(cls)
+        return any(
+            key_type in locator
+            for locator in self._iter_locators()
+            for key_type in key_types
+        )
 
     @property
     def is_locked(self) -> bool:
-        return any(broker.is_locked for broker in self.__brokers)
-
-    @property
-    def __brokers(self) -> Iterator[Broker]:
-        yield from self.__modules
-        yield self.__locator
+        return any(locator.is_locked(self) for locator in self._iter_locators())
 
     def injectable[**P, T](
         self,
@@ -443,16 +238,19 @@ class Module(Broker, EventListener):
         mode: Mode | ModeStr = Mode.get_default(),
     ) -> Any:
         def decorator(wp: Recipe[P, T]) -> Recipe[P, T]:
-            factory = extract_caller(self.make_injected_function(wp) if inject else wp)
-            injectable = cls(factory)  # type: ignore[arg-type]
             hints = on if ignore_type_hint else (wp, on)
-            updater = Updater.with_basics(hints, injectable, mode)
-            self.update(updater)
+            broker = (
+                DynamicInjectableBroker(cls, wp)
+                if inject
+                else StaticInjectableBroker.from_factory(cls, wp)
+            )
+            self.update_from(hints, broker, mode)
             return wp
 
         return decorator(wrapped) if wrapped else decorator
 
     singleton = partialmethod(injectable, cls=SingletonInjectable)
+    constant = partialmethod(injectable, cls=SingletonInjectable, inject=False)
 
     def scoped[**P, T](
         self,
@@ -497,30 +295,8 @@ class Module(Broker, EventListener):
 
     def should_be_injectable[T](self, wrapped: type[T] | None = None, /) -> Any:
         def decorator(wp: type[T]) -> type[T]:
-            injectable = ShouldBeInjectable(wp)
-            updater = Updater.with_basics(wp, injectable, Mode.FALLBACK)
-            self.update(updater)
-            return wp
-
-        return decorator(wrapped) if wrapped else decorator
-
-    def constant[**P, T](
-        self,
-        wrapped: Recipe[P, T] | None = None,
-        /,
-        *,
-        on: TypeInfo[T] = (),
-        mode: Mode | ModeStr = Mode.get_default(),
-    ) -> Any:
-        def decorator(wp: Recipe[P, T]) -> Recipe[P, T]:
-            recipe: Recipe[[], T] = alazy(wp) if iscoroutinefunction(wp) else lazy(wp)  # type: ignore[arg-type]
-            self.injectable(
-                recipe,
-                ignore_type_hint=True,
-                inject=False,
-                on=(wp, on),
-                mode=mode,
-            )
+            broker = StaticInjectableBroker(ShouldBeInjectable(wp))
+            self.update_from(wp, broker, Mode.FALLBACK)
             return wp
 
         return decorator(wrapped) if wrapped else decorator
@@ -532,28 +308,28 @@ class Module(Broker, EventListener):
         *,
         alias: bool = False,
         mode: Mode | ModeStr = Mode.get_default(),
-    ) -> Self:
-        hints = on if alias else (type(instance), on)
-        self.injectable(
-            lambda: instance,
-            ignore_type_hint=True,
-            inject=False,
-            on=hints,
-            mode=mode,
-        )
-        return self
+    ) -> T:
+        if not alias:
+            on = (type(instance), on)
+
+        elif not on:
+            raise ValueError("`on` must be provided when `alias` is `True`.")
+
+        broker = StaticInjectableBroker(ConstantInjectable(instance))
+        self.update_from(on, broker, mode)
+        return instance
 
     def reserve_scoped_slot[T](
         self,
-        cls: type[T],
+        cls: InputType[T],
         /,
         scope_name: str,
         *,
         mode: Mode | ModeStr = Mode.get_default(),
     ) -> SlotKey[T]:
         injectable = ScopedSlotInjectable(cls, scope_name)
-        updater = Updater.with_basics(cls, injectable, mode)
-        self.update(updater)
+        broker = StaticInjectableBroker(injectable)
+        self.update_from(cls, broker, mode)
         return injectable.key
 
     def inject[**P, T](
@@ -572,21 +348,23 @@ class Module(Broker, EventListener):
 
         return decorator(wrapped) if wrapped else decorator
 
-    @overload
-    def make_injected_function[**P, T](
-        self,
-        wrapped: Callable[P, T],
-        /,
-        threadsafe: bool | None = ...,
-    ) -> SyncInjectedFunction[P, T]: ...
+    if TYPE_CHECKING:  # pragma: no cover
 
-    @overload
-    def make_injected_function[**P, T](
-        self,
-        wrapped: Callable[P, Awaitable[T]],
-        /,
-        threadsafe: bool | None = ...,
-    ) -> AsyncInjectedFunction[P, T]: ...
+        @overload
+        def make_injected_function[**P, T](
+            self,
+            wrapped: Callable[P, T],
+            /,
+            threadsafe: bool | None = ...,
+        ) -> SyncInjectedFunction[P, T]: ...
+
+        @overload
+        def make_injected_function[**P, T](
+            self,
+            wrapped: Callable[P, Awaitable[T]],
+            /,
+            threadsafe: bool | None = ...,
+        ) -> AsyncInjectedFunction[P, T]: ...
 
     def make_injected_function[**P, T](
         self,
@@ -616,7 +394,7 @@ class Module(Broker, EventListener):
             wrapped,
             threadsafe,
         )
-        return factory.__inject_metadata__.acall
+        return factory.__injection_metadata__.acall
 
     async def afind_instance[T](
         self,
@@ -638,23 +416,25 @@ class Module(Broker, EventListener):
             injectable = self[cls]
             return injectable.get_instance()
 
-    @overload
-    async def aget_instance[T, Default](
-        self,
-        cls: InputType[T],
-        default: Default,
-        *,
-        threadsafe: bool | None = ...,
-    ) -> T | Default: ...
+    if TYPE_CHECKING:  # pragma: no cover
 
-    @overload
-    async def aget_instance[T](
-        self,
-        cls: InputType[T],
-        default: T = ...,
-        *,
-        threadsafe: bool | None = ...,
-    ) -> T: ...
+        @overload
+        async def aget_instance[T, Default](
+            self,
+            cls: InputType[T],
+            default: Default,
+            *,
+            threadsafe: bool | None = ...,
+        ) -> T | Default: ...
+
+        @overload
+        async def aget_instance[T](
+            self,
+            cls: InputType[T],
+            default: T = ...,
+            *,
+            threadsafe: bool | None = ...,
+        ) -> T: ...
 
     async def aget_instance[T, Default](
         self,
@@ -668,23 +448,25 @@ class Module(Broker, EventListener):
         except (KeyError, SkipInjectable):
             return default
 
-    @overload
-    def get_instance[T, Default](
-        self,
-        cls: InputType[T],
-        default: Default,
-        *,
-        threadsafe: bool | None = ...,
-    ) -> T | Default: ...
+    if TYPE_CHECKING:  # pragma: no cover
 
-    @overload
-    def get_instance[T](
-        self,
-        cls: InputType[T],
-        default: T = ...,
-        *,
-        threadsafe: bool | None = ...,
-    ) -> T: ...
+        @overload
+        def get_instance[T, Default](
+            self,
+            cls: InputType[T],
+            default: Default,
+            *,
+            threadsafe: bool | None = ...,
+        ) -> T | Default: ...
+
+        @overload
+        def get_instance[T](
+            self,
+            cls: InputType[T],
+            default: T = ...,
+            *,
+            threadsafe: bool | None = ...,
+        ) -> T: ...
 
     def get_instance[T, Default](
         self,
@@ -698,23 +480,25 @@ class Module(Broker, EventListener):
         except (KeyError, SkipInjectable):
             return default
 
-    @overload
-    def aget_lazy_instance[T, Default](
-        self,
-        cls: InputType[T],
-        default: Default,
-        *,
-        threadsafe: bool | None = ...,
-    ) -> Awaitable[T | Default]: ...
+    if TYPE_CHECKING:  # pragma: no cover
 
-    @overload
-    def aget_lazy_instance[T](
-        self,
-        cls: InputType[T],
-        default: T = ...,
-        *,
-        threadsafe: bool | None = ...,
-    ) -> Awaitable[T]: ...
+        @overload
+        def aget_lazy_instance[T, Default](
+            self,
+            cls: InputType[T],
+            default: Default,
+            *,
+            threadsafe: bool | None = ...,
+        ) -> Awaitable[T | Default]: ...
+
+        @overload
+        def aget_lazy_instance[T](
+            self,
+            cls: InputType[T],
+            default: T = ...,
+            *,
+            threadsafe: bool | None = ...,
+        ) -> Awaitable[T]: ...
 
     def aget_lazy_instance[T, Default](
         self,
@@ -727,26 +511,28 @@ class Module(Broker, EventListener):
             lambda instance=default: instance,
             threadsafe=threadsafe,
         )
-        metadata = function.__inject_metadata__.set_owner(cls)
+        metadata = function.__injection_metadata__.set_owner(cls)
         return SimpleAwaitable(metadata.acall)
 
-    @overload
-    def get_lazy_instance[T, Default](
-        self,
-        cls: InputType[T],
-        default: Default,
-        *,
-        threadsafe: bool | None = ...,
-    ) -> Invertible[T | Default]: ...
+    if TYPE_CHECKING:  # pragma: no cover
 
-    @overload
-    def get_lazy_instance[T](
-        self,
-        cls: InputType[T],
-        default: T = ...,
-        *,
-        threadsafe: bool | None = ...,
-    ) -> Invertible[T]: ...
+        @overload
+        def get_lazy_instance[T, Default](
+            self,
+            cls: InputType[T],
+            default: Default,
+            *,
+            threadsafe: bool | None = ...,
+        ) -> Invertible[T | Default]: ...
+
+        @overload
+        def get_lazy_instance[T](
+            self,
+            cls: InputType[T],
+            default: T = ...,
+            *,
+            threadsafe: bool | None = ...,
+        ) -> Invertible[T]: ...
 
     def get_lazy_instance[T, Default](
         self,
@@ -759,13 +545,28 @@ class Module(Broker, EventListener):
             lambda instance=default: instance,
             threadsafe=threadsafe,
         )
-        metadata = function.__inject_metadata__.set_owner(cls)
+        metadata = function.__injection_metadata__.set_owner(cls)
         return SimpleInvertible(metadata.call)
 
     def update[T](self, updater: Updater[T]) -> Self:
         self.__locator.update(updater)
         return self
 
+    def update_from[T](
+        self,
+        on: TypeInfo[T],
+        /,
+        broker: InjectableBroker[T],
+        mode: Mode | ModeStr,
+    ) -> Self:
+        updater = Updater(
+            classes=self.__build_key_types(on),
+            broker=broker,
+            mode=Mode(mode),
+        )
+        self.update(updater)
+        return self
+
     def init_modules(self, *modules: Module) -> Self:
         for module in tuple(self.__modules):
             self.stop_using(module)
@@ -835,20 +636,14 @@ class Module(Broker, EventListener):
         return self
 
     def unlock(self) -> Self:
-        event = UnlockCalled()
-
-        with self.dispatch(event, lock_bypass=True):
-            self.unsafe_unlocking()
+        for locator in self._iter_locators():
+            locator.unlock(self)
 
         return self
 
-    def unsafe_unlocking(self) -> None:
-        for broker in self.__brokers:
-            broker.unsafe_unlocking()
-
     async def all_ready(self) -> None:
-        for broker in self.__brokers:
-            await broker.all_ready()
+        for locator in self._iter_locators():
+            await locator.all_ready(self)
 
     def add_logger(self, logger: Logger) -> Self:
         self.__loggers.append(logger)
@@ -867,9 +662,8 @@ class Module(Broker, EventListener):
         return self.dispatch(self_event)
 
     @contextmanager
-    def dispatch(self, event: Event, *, lock_bypass: bool = False) -> Iterator[None]:
-        if not lock_bypass:
-            self.__check_locking()
+    def dispatch(self, event: Event) -> Iterator[None]:
+        self.__check_locking()
 
         with self.__channel.dispatch(event):
             try:
@@ -877,6 +671,12 @@ class Module(Broker, EventListener):
             finally:
                 self.__debug(event)
 
+    def _iter_locators(self) -> Iterator[Locator]:
+        for module in self.__modules:
+            yield from module._iter_locators()
+
+        yield self.__locator
+
     def __debug(self, message: object) -> None:
         for logger in self.__loggers:
             logger.debug(message)
@@ -908,6 +708,20 @@ class Module(Broker, EventListener):
     def default(cls) -> Module:
         return cls.from_name("__default__")
 
+    @staticmethod
+    def __build_key_types(input_cls: Any) -> frozenset[Any]:
+        config = MatchingTypesConfig(ignore_none=True)
+        return frozenset(
+            itertools.chain.from_iterable(
+                iter_matching_types(cls, config) for cls in get_return_types(input_cls)
+            )
+        )
+
+    @staticmethod
+    def __matching_key_types(input_cls: Any) -> tuple[Any, ...]:
+        config = MatchingTypesConfig(with_origin=True, with_type_alias_value=True)
+        return matching_types(input_cls, config)
+
 
 def mod(name: str | None = None, /) -> Module:
     if name is None:
@@ -1129,24 +943,24 @@ class InjectMetadata[**P, T](Caller[P, T], EventListener):
             task()
 
 
-class InjectedFunction[**P, T](ABC):
-    __slots__ = ("__dict__", "__inject_metadata__")
+class InjectedFunction[**P, T](HiddenCaller[P, T], ABC):
+    __slots__ = ("__dict__", "__injection_metadata__")
 
-    __inject_metadata__: InjectMetadata[P, T]
+    __injection_metadata__: InjectMetadata[P, T]
 
     def __init__(self, metadata: InjectMetadata[P, T]) -> None:
         update_wrapper(self, metadata.wrapped)
-        self.__inject_metadata__ = metadata
+        self.__injection_metadata__ = metadata
 
     def __repr__(self) -> str:  # pragma: no cover
-        return repr(self.__inject_metadata__.wrapped)
+        return repr(self.__injection_metadata__.wrapped)
 
     def __str__(self) -> str:  # pragma: no cover
-        return str(self.__inject_metadata__.wrapped)
+        return str(self.__injection_metadata__.wrapped)
 
-    @abstractmethod
-    def __call__(self, /, *args: P.args, **kwargs: P.kwargs) -> T:
-        raise NotImplementedError
+    @property
+    def __injection_hidden_caller__(self) -> Caller[P, T]:
+        return self.__injection_metadata__
 
     def __get__(
         self,
@@ -1159,7 +973,7 @@ class InjectedFunction[**P, T](ABC):
         return MethodType(self, instance)
 
     def __set_name__(self, owner: type, name: str) -> None:
-        self.__inject_metadata__.set_owner(owner)
+        self.__injection_metadata__.set_owner(owner)
 
 
 class AsyncInjectedFunction[**P, T](InjectedFunction[P, Awaitable[T]]):
@@ -1170,23 +984,11 @@ class AsyncInjectedFunction[**P, T](InjectedFunction[P, Awaitable[T]]):
         markcoroutinefunction(self)
 
     async def __call__(self, /, *args: P.args, **kwargs: P.kwargs) -> T:
-        return await (await self.__inject_metadata__.acall(*args, **kwargs))
+        return await (await self.__injection_metadata__.acall(*args, **kwargs))
 
 
 class SyncInjectedFunction[**P, T](InjectedFunction[P, T]):
     __slots__ = ()
 
     def __call__(self, /, *args: P.args, **kwargs: P.kwargs) -> T:
-        return self.__inject_metadata__.call(*args, **kwargs)
-
-
-def extract_caller[**P, T](
-    function: Callable[P, T] | Callable[P, Awaitable[T]],
-) -> Caller[P, T]:
-    if iscoroutinefunction(function):
-        return AsyncCaller(function)
-
-    elif isinstance(function, InjectedFunction):
-        return function.__inject_metadata__
-
-    return SyncCaller(function)  # type: ignore[arg-type]
+        return self.__injection_metadata__.call(*args, **kwargs)
